/*
 * piQa Services Collection
 * Created on Apr 23, 2005
 * Copyright � 2005 Ronald D. Grindle
 */
package RTObj;

import com.qanavigation.Basics.BTools;
import com.qanavigation.Basics.Rslt;
import com.qanavigation.Basics.Rslt2DArray;
import com.qanavigation.Basics.RsltString;
import com.qanavigation.Basics.SFormatter;
import com.qanavigation.QNservices.QNDefs;
import com.qanavigation.XMLBasics.RsltXMLDoc;
import com.qanavigation.XMLBasics.RsltXMLNode;
import com.qanavigation.XMLDB.XMLDB;

/**
 * Fix Document Builder
 */
public class FixDocBld {
	
	private XMLDB m_xDB;
	
	//-------------------------------------
	public FixDocBld( XMLDB p_xDB ) {
		
		m_xDB = p_xDB;
	}

	/**
	 * Builds a fix document for a problem report. 
	 * <p>Assumes application and title etc. are the same for the fix and the PR.
	 * @param p_Appl
	 * @param p_Assgn
	 * @param p_Fctn
	 * @param p_FixDesc is encodeHTML()
	 * @param p_FixImp  is encodeHTML()
	 * @param p_Release
	 * @param p_Status
	 * @param p_FixTstReq  is encodeHTML()
	 * @param p_Title
	 * @param p_FixLst is encodeHTML()
	 * @param p_PRUid
	 * @return
	 */
	public RsltXMLDoc bldFinalFixDoc( String p_Appl, String p_Assgn, String p_Fctn, String p_FixDesc, String p_FixImp, 
			String p_Release, String p_Status, String p_FixTstReq, String p_Ticket, String p_Title, Rslt2DArray p_FixLst, String p_PRUid ) {
		
		int i, x;
		
		RsltXMLNode FXNode;
		RsltXMLNode aNode;
		RsltXMLNode bNode;
		
		RsltXMLDoc xDoc; 

		String[] fix;
		
		String dt;
	
		/*
			<!ELEMENT Fix (RefProbRep*, Desc?, FixLst, AddInfo*, LogEntry*)>
				<!ATTLIST Fix
					Appl					CDATA #IMPLIED
					Assignd				CDATA #IMPLIED
					DateTime			NMTOKEN #REQUIRED
					DateTimeFix		NMTOKEN #IMPLIED
					DateTimeShip	NMTOKEN #IMPLIED
					Fctn					CDATA #IMPLIED
					FixStat				(as|at|bd|cl|ct|es|ev|fx|qr|qt|re|rv|sh|tf|ts|wf) "as"
					LastMod				NMTOKEN #IMPLIED
					LastModBy			CDATA #IMPLIED
					Lock					NMTOKEN #IMPLIED
					PlanRel				NMTOKEN #IMPLIED
					ProbRel				NMTOKEN #IMPLIED
					ShipRel				NMTOKEN #IMPLIED
					Title					CDATA #REQUIRED
					UID						NMTOKEN #IMPLIED					
				>
		*/

		xDoc = new RsltXMLDoc( "", QNDefs.c_NodeFix, QNDefs.c_FixDTD, null );
		
		try {
				
			if( xDoc.isOK() ) {
				
				FXNode = xDoc.getTopNode();
		
				//<?xml version="1.0"?>
				//FXNode.createXmlDecl( "1.0", "", "" );
	
				//<!-- Generated by QA Navigator Fix Proc... -->
				String genCom = QNDefs.c_PhrGenBy + this.getClass().getName();
				FXNode.addComment( genCom );
		
				FXNode.addAttribute( "Appl", p_Appl );
		
				FXNode.addAttribute( "Assignd", p_Assgn );
				
				dt = BTools.getNowISODateTimeSec();
					
				FXNode.addAttribute( "DateTime", dt );
				
				FXNode.addAttribute( "DateTimeFix", dt );
		
				FXNode.addAttribute( "Fctn", p_Fctn );
		
				FXNode.addAttribute( "FixStat", p_Status );
		
				//FXNode.addAttribute( "PlanRel", p_TrgtRelease );
		
				FXNode.addAttribute( "ProbRel", p_Release );
				
				FXNode.addAttribute( "Ticket", p_Ticket );
				
				FXNode.addAttribute( "Title", p_Title );
				
				if( FXNode.isOK() ) {
					
					/*
						<!ELEMENT RefProbRep (EMPTY)>
							<!ATTLIST RefProbRep
								Appl		CDATA #IMPLIED
								Fctn		CDATA #IMPLIED
								Title		CDATA #REQUIRED
								RefUID	NMTOKEN #REQUIRED
								UID			NMTOKEN #IMPLIED
							>
					*/
			
					aNode = FXNode.appNode( "RefProbRep" );
					
					aNode.addAttribute( "Appl", p_Appl );
					aNode.addAttribute( "Fctn", p_Fctn );
					aNode.addAttribute( "Title", p_Title );				
					aNode.addAttribute( "RefUID", p_PRUid );
					
					aNode = FXNode.appTxtNode( "Desc", SFormatter.encodeHTML( p_FixDesc ) );
					
					aNode = FXNode.appTxtNode( "TestReq", SFormatter.encodeHTML( p_FixTstReq ) );
					
					aNode = FXNode.appTxtNode( "FixImp", SFormatter.encodeHTML( p_FixImp ) );					
					
					/*
						<!ELEMENT FixLst (FixEntry*)>
							<!ATTLIST FixLst
								UID				NMTOKEN #IMPLIED
							>
					*/	

					aNode = FXNode.appNode( "FixLst" );
					
					x = p_FixLst.getRowNum();

					for( i = 0; i < x; ++i ) {

						fix = p_FixLst.getRow( i );
						 
						bNode = aNode.appNode( "FixEntry" );
						
						//branch, date, file, path, rel, ""
						
						bNode.addAttrAsHTML( "Author", fix[0] );
						bNode.addAttrAsHTML( "Branch", fix[1] );
						bNode.addAttrAsHTML( "Cmnt", fix[2] );				
						bNode.addAttribute( "DateTime", fix[3] );
						bNode.addAttrAsHTML( "Label", fix[4] );
						bNode.addAttrAsHTML( "Name", fix[5] );
						bNode.addAttrAsHTML( "Path", fix[6] );
						bNode.addAttrAsHTML( "Rel", fix[7] );
						bNode.addAttrAsHTML( "Section", fix[8] );
						bNode.addAttrAsHTML( "SrcSrv", fix[9] );						
					}					
				}
				
				xDoc.setRsltStat( FXNode );
			}		
		}
		catch( Exception excp ) {
			
			xDoc.setRsltByExcp( excp );
		}
		
		return xDoc;
	}

	//-------------------------------------	
	/**
	 * 
	 */
	public RsltString createNewFXdoc( String p_Appl, String p_Author, String p_Ticket, String p_Fctn, String p_FixDesc, String p_FixImp, 
			String p_Release, String p_Status, String p_FixTstReq, String p_Title, Rslt2DArray p_FixLst, String p_PRUid ) {
		
		RsltString res = new RsltString( true );
		
		RsltXMLDoc xDoc;
		
		try {
		
			xDoc = bldFinalFixDoc( p_Appl, p_Author, p_Fctn, p_FixDesc, p_FixImp, p_Release, p_Status, p_FixTstReq, p_Ticket, p_Title, p_FixLst, p_PRUid );
		
			if( res.checkRslt( xDoc ) ) {
			
				res = m_xDB.importXML( xDoc.m_xDoc );
			}
		}
		catch( Exception excp ) {
				
			res.setRsltByExcp( excp );
		}

		return res;
	}
	
	/**
	 * 
	 * @param p_FixUID
	 * @param p_Author
	 * @param p_FixDesc
	 * @param p_FixImp
	 * @param p_FixTstReq
	 * @param p_FixLst
	 * @return
	 */
	public Rslt updFXdoc( String p_FixUID, String p_Author, String p_FixDesc, String p_FixImp, String p_FixTstReq, Rslt2DArray p_FixLst ) {
		
		Rslt res = new Rslt( true );
		
		int i, x;
		
		RsltXMLNode nodeFixLst;
		RsltXMLNode aNode;
		
		String[] row;
		
		res.checkRslt( m_xDB.setAttrVal( p_FixUID, QNDefs.c_AttrAssignd, p_Author ) );
		
		if( res.isOK() ) {

			res.checkRslt( m_xDB.setSubNodeVal( p_FixUID, QNDefs.c_NodeDesc, SFormatter.encodeHTML( p_FixDesc ) ) );
			res.checkRslt( m_xDB.setSubNodeVal( p_FixUID, QNDefs.c_NodeFixImp, SFormatter.encodeHTML( p_FixImp ) ) );
			res.checkRslt( m_xDB.setSubNodeVal( p_FixUID, QNDefs.c_NodeTestReq , SFormatter.encodeHTML( p_FixTstReq ) ) );
		}
		
		if( res.isOK() ) {

			res.checkRslt( m_xDB.deleteSubTree( p_FixUID, QNDefs.c_NodeFixLst, true ) );
		}
		
		if( res.isOK() ) {			

			nodeFixLst = new RsltXMLNode( QNDefs.c_NodeFixLst );

			if( res.checkRslt( nodeFixLst ) ) {
				
				x = p_FixLst.getRowNum();
	
				for( i = 0; i < x; ++i ) {
	
					row = p_FixLst.getRow( i );
	
					aNode = nodeFixLst.appNode( "FixEntry" );
	
					//branch, date, file, path, rel, ""
	
					aNode.addAttrAsHTML( "Author", row[0] );
					aNode.addAttrAsHTML( "Branch", row[1] );
					aNode.addAttrAsHTML( "Cmnt", row[2] );				
					aNode.addAttribute( "DateTime", row[3] );
					aNode.addAttrAsHTML( "Label", row[4] );
					aNode.addAttrAsHTML( "Name", row[5] );
					aNode.addAttrAsHTML( "Path", row[6] );
					aNode.addAttrAsHTML( "Rel", row[7] );
					aNode.addAttrAsHTML( "Section", row[8] );
					aNode.addAttrAsHTML( "SrcSrv", row[9] );						
				}
			}

			res.checkRslt( m_xDB.appSubTree( p_FixUID, nodeFixLst, false ) );
		}
		
		if( res.notOK() ) {
			
			res.addRsltAddFailInfo( "FixDocBld", "updFXdoc", "", "" );
		}
		
		return res;		
	}
}
